@@ -6,7 +6,6 @@
 #  Created by Alan D Snow, 2016.
 #  BSD 3-Clause
 
-from datetime import datetime
 from distutils.spawn import find_executable
 from glob import glob
 import logging
@@ -23,7 +22,6 @@ except ImportError:
     pass
 
 from ..lib import db_tools as dbt
-from ..orm import ProjectFile
 from .event import EventMode, LongTermMode
 from ..util.context import tmp_chdir
 
@@ -245,6 +243,7 @@ class GSSHAFramework(object):
         self.lsm_data_var_map_array = lsm_data_var_map_array
         self.lsm_precip_data_var = lsm_precip_data_var
         self.lsm_precip_type = lsm_precip_type
+        self.precip_interpolation_type = precip_interpolation_type
         self.output_netcdf = output_netcdf
         self.write_hotstart = write_hotstart
         self.read_hotstart = read_hotstart
@@ -444,7 +443,8 @@ class GSSHAFramework(object):
         """
         if self._prepare_lsm_gag:
             self.event_manager.prepare_gag_lsm(self.lsm_precip_data_var,
-                                               self.lsm_precip_type)
+                                               self.lsm_precip_type,
+                                               self.precip_interpolation_type)
             self.simulation_modified_input_cards.append("PRECIP_FILE")
         else:
             log.info("Gage file preparation skipped due to missing parameters ...")
@@ -610,7 +610,14 @@ class GSSHAFramework(object):
                     if gssha_card.value:
                         updated_value = gssha_card.value.strip('"').strip("'")
                         if updated_value:
-                            if os.path.exists(updated_value):
+                            if gssha_card.name == "READ_CHAN_HOTSTART":
+                                # there are two required files
+                                # the .dht and .qht
+                                if os.path.exists(updated_value + '.dht') \
+                                        and os.path.exists(updated_value + '.qht'):
+                                    updated_path = os.path.join("..", os.path.basename(updated_value))
+                                    gssha_card.value = '"{0}"'.format(updated_path)
+                            elif os.path.exists(updated_value):
                                 updated_path = os.path.join("..", os.path.basename(updated_value))
                                 gssha_card.value = '"{0}"'.format(updated_path)
                             elif gssha_card.name == '#INDEXGRID_GUID':
